/** Main class of dfrac.
 *  Defines the main frame as well as most of the program logic, all the menus
 *  and the toolbar.
 *
 */

package Control;
import util.*;
import DAL.*;
import PL.*;
import java.net.URL;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.text.*;
import java.io.*;
import java.beans.*;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.image.codec.jpeg.JPEGCodec;

/** Main Class of dfrac, implements most program logic as well
 * as all menus and toolbar
 */
public class ClDFrac extends JFrame {
    
    /** the main Frame (i.e. this) */
    public static ClDFrac mwTopWnd;
    
    /** the Fractal Calculation Controler */
    private ClCalcControl moCalcControler;
    
    /** the main pane */
    private ClDFracWidget moMainWidget;
    
    /** status TextField */
    public static JTextField mtStatusText;
    
    /** the normal toolbar */
    private JToolBar moToolBar;
    
    /** NMax slider */
    private JSlider moNMaxSlider;
    
    /** JSpinner (change fields) */
    private JSpinner[] maoSpinners=null;
    
    /** Wether Spinners shall react on value changes, or not */
    private boolean mbSpinnerActionHandling = true;
    
    private IFrac moDAL = null;
    
    /** Constructor of ClDFrac
     *  @param pstrArgs String Array with files to open */
    public ClDFrac(String[] pstrArgs){
        super(ClDFracVersion.VersionString());
        
        int i;
        mwTopWnd = this;
        
        //set some accessible desc
        this.getAccessibleContext().setAccessibleDescription(
        "A Tool for distributed Fractal Calculation.");
        
        try{ CreateMenuAndToolBars(); }
        catch(EDFracError ei){
            eMsg("Internal Error while initializing menus and toolbar",ei,true);
            System.exit(1);
        }
        catch(Throwable ex){
            eMsg("Error while initializing menus and toolbar",ex,false);
            System.exit(1);
        }
        
        
        int nmax = 100;
        
        try{
            moDAL = new ClFrac(nmax, 0.0, 0.0, 100);
        } catch(Exception e){ }
        
        moCalcControler = new ClCalcControl(moDAL,this);
        
        moMainWidget = new ClDFracWidget(mtStatusText,moCalcControler.getReadyPiecesQueue());
        //moMainWidget.setUpdatedPiecesQueue(
        moMainWidget.setBorder(BorderFactory.createLoweredBevelBorder());
        this.getContentPane().add(moMainWidget, BorderLayout.CENTER);
        getGlassPane().addMouseListener(new MouseAdapter() {});
        
        //Display the window
        this.pack();
        this.setSize(800, 600);
        
        this.setVisible(true);
        // wait until all windows got arranged
               /* try{
                    Thread.sleep(500);
                } catch(Exception e){}*/
        moMainWidget.setSelChangedListener((ClFrac)moDAL);
        moMainWidget.setDALObj(moDAL);
        try{
            moDAL.setBoundaries( moMainWidget.getWidth(),
            moMainWidget.getHeight() );
        }catch(EDFracError err){
            DFracLog.log(1,"Error: "+err.getMessage());
        }
        moCalcControler.distributedCalc();
    }
    
    
    /** Creates the Menus and the toolbar at startup and anytime the language is changed
     *  @throws EDFracError if something went wrong  */
    private void CreateMenuAndToolBars() throws EDFracError{
        byte i;
        
        //reset main Frame
        this.getContentPane().removeAll();
        
        //ToolBar
        JToolBar moToolBar = new JToolBar();
        moToolBar.setFloatable(false);
        
        //menu
        JMenuBar moMenuBar = new JMenuBar();
        
        //top level entries
        JMenu myMenuFile = new JMenu("File");
        int it = ((int) ( (("MenuFile_Mnemonic").length()>1)?"F":"MenuFile_Mnemonic" ).charAt(0) );
        myMenuFile.setMnemonic( it );
        
        JMenu myMenuOption = new JMenu((String)"Option");
        it = ((int) ( (("MenuOption_Mnemonic").length()>1)?"O":"MenuOption_Mnemonic" ).charAt(0) );
        myMenuOption.setMnemonic(it);
        
        JMenu myMenuHelp = new JMenu((String)"Help");
        it = ((int) ( (("MenuHelp_Mnemonic").length()>1)?"H":"MenuHelp_Mnemonic" ).charAt(0) );
        myMenuHelp.setMnemonic( it );
        
        moMenuBar.add(myMenuFile);
        moMenuBar.add(myMenuOption);
        moMenuBar.add(myMenuHelp);
        
        //sub entries and moToolBar buttons
        
        //File-menu
        
        final Action vExitAction;
        
        vExitAction = new ExitAction();
        Action[] Actions1 = {new SaveImgAction()};
        loadActions(Actions1, myMenuFile, moToolBar);
        myMenuFile.add(new JMenuItem(vExitAction));
        
        //Option Menu
        
        Action[] Actions2 = {new ResetAction(), new ShowControlerAction()};
        loadActions(Actions2,myMenuOption,moToolBar);
        
        //Help Menu
        moToolBar.addSeparator();
        Action[] Actions3 = {new InfoAction()};
        loadActions(Actions3,myMenuHelp,moToolBar);
        
        
        
        //Status Bar
        JPanel statusPane = new JPanel(new GridLayout(1, 1));
        statusPane.setPreferredSize(new Dimension(700,18));
        
        mtStatusText = new JTextField("ready");
        mtStatusText.setPreferredSize(new Dimension(700,18));
        mtStatusText.setEditable(false);
        mtStatusText.setBorder(BorderFactory.createLoweredBevelBorder());
        
        statusPane.add(mtStatusText);
        
        // nMax, zoom amd center Spinner
        maoSpinners = new JSpinner[4];
        String labels[] = {"nMax: ", "Zoom: ", "Center:", "x", "y"};
        int nMax = 100, zoom=100;
        double dXVal=0.0, dYVal=0.0;
        SpinnerModel nMaxModel = new SpinnerNumberModel(nMax, //initial value
        nMax - 100, //min
        nMax + 999900, //max
        10);                //step
        SpinnerModel zoomModel = new SpinnerNumberModel(zoom, //initial value
        zoom - 100, //min
        Integer.MAX_VALUE, //max
        100);                //step
        SpinnerModel xValModel = new SpinnerNumberModel(dXVal, //initial value
        dXVal - 2.0, //min
        dXVal + 2.0, //max
        0.01);                //step
        SpinnerModel yValModel = new SpinnerNumberModel(dXVal, //initial value
        dXVal - 2.0, //min
        dXVal + 2.0, //max
        0.01);                //step
        moToolBar.addSeparator();
        maoSpinners[0]= addLabeledSpinner(moToolBar, labels[0], nMaxModel);
        maoSpinners[0].addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent e){
                if(mbSpinnerActionHandling){
                    int i = ((Integer)(((JSpinner)e.getSource()).getValue())).intValue();
                    int n=1;
                    if(i>=1000){
                        Object[] options = {"Sure, I have the time.",
                        "No, sorry, didn't know about that!"};
                        n = JOptionPane.showOptionDialog(
                        mwTopWnd,
                        i+" Iterations are really a lot of work for me to do\n"+
                        "and enough time for you to have some cups of coffe.\n"+
                        " Do you really want me to calculate that?!",
                        "Excuse me for asking...",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE,
                        null,
                        options,
                        options[0]);
                    }
                    if(i<1000 || n==0){
                        DFracLog.log(3,"nMaxSpinner: "+i);
                        try{
                            moDAL.setNMax(i);
                        }catch (Exception ex) {
                            DFracLog.log(1,"Error during nMaxChange: "+
                            ex.getMessage());
                        }
                    }
                    else{
                        mbSpinnerActionHandling=false;
                        maoSpinners[0].setValue(new Integer(moDAL.getNMax()));
                        mbSpinnerActionHandling=true;
                    }
                }
            }
        }
        );
        moToolBar.addSeparator();
        maoSpinners[1] = addLabeledSpinner(moToolBar, labels[1], zoomModel);
        maoSpinners[1].addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent e){
                if(mbSpinnerActionHandling){
                    Integer i = (Integer)(((JSpinner)e.getSource()).getValue());
                    DFracLog.log(3,"zoomSpinner: "+i.intValue());
                    try{
                        moDAL.setZoom(i.intValue());
                    }catch (Exception ex) {
                        DFracLog.log(1,"Error during ZoomChange: "
                        +ex.getMessage());
                    }
                }
            }
        }
        );
        moToolBar.addSeparator();
        JLabel l = new JLabel(labels[2]);
        moToolBar.add(l);
        moToolBar.addSeparator();
        maoSpinners[2] = addLabeledSpinner(moToolBar, labels[3], xValModel);
        maoSpinners[2].addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent e){
                if(mbSpinnerActionHandling){
                    Double dCenterX = (Double)(((JSpinner)e.getSource()).getValue());
                    Double dCenterY = (Double)maoSpinners[3].getValue();
                    DFracLog.log(3,"centerXSpinner: "+dCenterX.doubleValue());
                    try{
                        moDAL.setCenter(dCenterX.doubleValue(),
                        dCenterY.doubleValue());
                    }catch (Exception ex) {
                        DFracLog.log(1,"Error during centerXChange: "
                        +ex.getMessage());
                    }
                }
            }
        }
        );
        moToolBar.addSeparator();
        maoSpinners[3] = addLabeledSpinner(moToolBar, labels[4], yValModel);
        maoSpinners[3].addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent e){
                if(mbSpinnerActionHandling){
                    Double dCenterX = (Double)maoSpinners[2].getValue();
                    Double dCenterY = (Double)(((JSpinner)e.getSource()).getValue());
                    DFracLog.log(3,"centerYSpinner: "+dCenterY.doubleValue());
                    try{
                        moDAL.setCenter(dCenterX.doubleValue(),
                        dCenterY.doubleValue());
                    }catch (Exception ex) {
                        DFracLog.log(1,"Error during centerYChange: "+
                        ex.getMessage());
                    }
                }
            }
        }
        );
        
        // Color-Effect chooser (combobox)
        String[] colorEffectStrings = { "smooth Blue2Black",
        "smooth Rainbow",
        "snoop",
        "gain colored",
        "gain blue2black"};
        JComboBox colEffectChooser = new JComboBox(colorEffectStrings);
        colEffectChooser.setSelectedIndex(0);
        colEffectChooser.addActionListener(
        new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                byte colEffectName = (byte)cb.getSelectedIndex();
                moMainWidget.changeColorEffect(colEffectName);
            }
        }
        );
        moToolBar.addSeparator();
        moToolBar.add(colEffectChooser);
        
        //Set the menu bar and add moToolBar
        JPanel pane = new JPanel(new BorderLayout());
        
        pane.add(moToolBar, BorderLayout.NORTH);
        
        this.setJMenuBar(moMenuBar);
        this.getContentPane().add(pane, BorderLayout.NORTH);
        this.getContentPane().add(statusPane, BorderLayout.SOUTH);
        
        this.addWindowListener(
        new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                //call the method of ExitAction class
                vExitAction.actionPerformed(
                new ActionEvent(this,ActionEvent.ACTION_PERFORMED,null));
            }
        });
    }
    
    /*public void setSpinnerActionHandling(boolean bDoActionHandling){
        mbSpinnerActionHandling = bDoActionHandling;
    }*/
    
    public void setSpinners(){
        DFracLog.log(3,"Main: setting spinners");
        mbSpinnerActionHandling=false;
        maoSpinners[1].setValue(new Integer(moDAL.getZoom()));
        maoSpinners[2].setValue(new Double(moDAL.getCenterX()));
        maoSpinners[3].setValue(new Double(moDAL.getCenterY()));
        mbSpinnerActionHandling=true;
        
    }
    
    static protected JSpinner addLabeledSpinner(Container c,
    String label,
    SpinnerModel model) {
        JLabel l = new JLabel(label);
        c.add(l);
        JSpinner spinner = new JSpinner(model);
        l.setLabelFor(spinner);
        c.add(spinner);
        return spinner;
    }
    
    /** called from ClCalcControl to repaint current DAL */
    public void repaintCurrentDAL(){
        moMainWidget.repaint();
    }
    
    public void forcedRepaintCurrentDAL(){
        moMainWidget.forcedRepaint();
    }
    
    //helper funcs
    
    /** adds the specified Action to given menu and moToolBar
     *  @param paActions Array of Actions to add
     *  @param pmMenu Menu to add the Actions to
     *  @param pToolBar Toolbar to add the Actions to */
    private void loadActions(Action[] paActions, JMenu pmMenu, JToolBar pToolBar){
        byte i;
        JButton b;
        JMenuItem jmi;
        
        for (i=0;i<paActions.length;i++){
            if (paActions[i]==null){ pmMenu.addSeparator(); pToolBar.addSeparator();}
            else{
                b = new JButton(paActions[i]);
                b.setText("");
                b.setMnemonic(0);
                jmi = new JMenuItem(paActions[i]);
                pmMenu.add(jmi);
                pToolBar.add(b);
            }
        }
    }
    /** Returns an ImageIcon, or null if the path was invalid.
     *  @param pstrImageName Name of the Image without trailing .gif
     *  @return newly created ImageIcon */
    public static ImageIcon createIcon(String pstrImageName) {
        ImageIcon ic = null;
        try {
            URL iconLocation = ClDFrac.class.getResource("/res/"+pstrImageName+"16.gif");
            Image image = Toolkit.getDefaultToolkit().createImage(iconLocation);
            ic = new ImageIcon(image);
        }
        catch(Exception e){  }
        return ic;
    }
    
    /** shows an error message
     *  @param pstrError the Error String to show
     *  @param peException the Exception that occured
     *  @param pbInternal if its an InternalError */
    private void eMsg(String pstrError,Throwable peException,boolean pbInternal){
        peException.printStackTrace();
        JOptionPane.showMessageDialog(this,
        java.text.MessageFormat.format(pstrError+":\n{0}", new Object[] { peException.toString() }),
        (pbInternal)?"Internal Error!":"Error!",
        JOptionPane.ERROR_MESSAGE );
    }
    
    
    /** Action class for "Exit" */
    class ExitAction extends AbstractAction {
        /** Constructor for private class */
        public ExitAction() {
            super((String)"Exit", createIcon("Exit"));
            putValue(SHORT_DESCRIPTION, "Exits dFrac");
            putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_X));
            putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_X,InputEvent.CTRL_MASK));
        }
        /** "main" method of this private class, does all the work ;)
         * @param e the ActionEvent assocciated with the action performed (Button etc.) */
        public void actionPerformed(ActionEvent e) {
            byte i;
            
            try{
                moCalcControler.stop();
                moCalcControler.kill();
                boolean bExit=true;
                mwTopWnd.dispose();
                System.exit(0);
            }
            catch(Throwable ex){
                eMsg("Error while exiting dFrac",ex,false);
                System.exit(1);
            }
        }
        
        
    }
    
    //reset Menu
    /** Action class for "Show about dialog" */
    class ResetAction extends AbstractAction {
        /** Constructor for private class */
        public ResetAction() {
            super((String)"reset Fractal", createIcon("Zoom"));
            putValue(SHORT_DESCRIPTION, "resets the Fractal");
            putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_R));
        }
        /** "main" method of this private class, does all the work ;)
         * @param e the ActionEvent assocciated with the action performed (Button etc.) */
        public void actionPerformed(ActionEvent e) {
            Rectangle bounds=moMainWidget.getBounds();
            moDAL.reset();
            try{ moDAL.setBoundaries((int)bounds.getWidth(), (int)bounds.getHeight()); }
            catch(Exception ex){ DFracLog.log(1,"main:Error while resetting:"+ex.getMessage()); }
        }
    }
    
    //Help Menu
    /** Action class for "Show about dialog" */
    class InfoAction extends AbstractAction {
        /** Constructor for private class */
        public InfoAction() {
            super((String)"About...", createIcon("About"));
            putValue(SHORT_DESCRIPTION, "Displays the about Dialog");
            putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_B));
        }
        /** "main" method of this private class, does all the work ;)
         * @param e the ActionEvent assocciated with the action performed (Button etc.) */
        public void actionPerformed(ActionEvent e) {
            try{ ClAboutDialog ad = new ClAboutDialog(mwTopWnd); }
            catch(EDFracError ei){}
        }
    }
    
    /** Action class for "Save Image" */
    class SaveImgAction extends AbstractAction {
        /** Constructor for private class */
        public SaveImgAction() {
            super("Save Image ...", createIcon("SaveAs"));
            putValue(SHORT_DESCRIPTION, "Saves the current Image");
            putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
            putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_S,InputEvent.CTRL_MASK));
        }
        /** "main" method of this private class, does all the work ;)
         * @param e the ActionEvent assocciated with the action performed (Button etc.) */
        public void actionPerformed(ActionEvent e) {
            mwTopWnd.saveImage();
        }
    }
     
    /** Action class for "Show Controler" */
    class ShowControlerAction extends AbstractAction {
        /** Constructor for private class */
        public ShowControlerAction() {
            super("Show/hide CalcControler ...", createIcon("Contr"));
            putValue(SHORT_DESCRIPTION, "Shows/hides the CalcControler Window");
            putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
            
        }
        /** "main" method of this private class, does all the work ;)
         * @param e the ActionEvent assocciated with the action performed (Button etc.) */
        public void actionPerformed(ActionEvent e) {
            moCalcControler.setVisible(!moCalcControler.isVisible());
        }
    }
    
    private void saveImage(){
        String FileName = null;
        JFileChooser fc = new JFileChooser();
        fc.setDialogType( JFileChooser.SAVE_DIALOG );
        fc.setMultiSelectionEnabled( false );
        fc.setDialogTitle("Save Image...");
        
        if(fc.showSaveDialog(mwTopWnd) == JFileChooser.APPROVE_OPTION){
            FileName = fc.getSelectedFile().getAbsolutePath();
        }
        else return;
        
        try {
            if (!FileName.endsWith(".jpg")) FileName+=".jpg";
            // write image
            
            // Encode as a JPEG
            BufferedImage oImg=moMainWidget.getImage();
            FileOutputStream fos = new FileOutputStream(FileName);
            JPEGImageEncoder jpeg = JPEGCodec.createJPEGEncoder(fos);
            jpeg.encode(oImg);
            fos.close();
        } catch (IOException err) {  JOptionPane.showMessageDialog(mwTopWnd,
        MessageFormat.format("Error while writing to {0}:\n{1}",
        new Object[] { FileName, err.getMessage() }),
        "Error!", JOptionPane.ERROR_MESSAGE ); }
        
    }
   
}
