/** Class that represents a Fractal as defined by IFrac
 *
 */

package DAL;

import util.*;
import DAL.*;
import java.util.Vector;
import java.util.Iterator;

public class ClFrac implements DAL.IFrac,ISelChangedListener {
    private int mNMax=0;
    private int mnWidth=0;
    private int mnHeight=0;
    private double mdCenterX=0.0;
    private double mdCenterY=0.0;
    private int mnZoom=100;
    private ClFracPiece[] maFracPieces=null;
    private IFracChangedListener miChangeListener;
    private IFracUpdatedListener miUpdateListener;

    /** create a new, empty Fractal */
    public ClFrac(int nmax, double dCenter_x, double dCenter_y, int nZoom)
    throws EDFracError {
        mNMax = nmax;
        mdCenterX=dCenter_x;
        mdCenterY=dCenter_y;
        mnZoom = nZoom;
    }

    public void setChangeListener(IFracChangedListener ifcl){
        miChangeListener=ifcl;
    }
    public void setUpdateListener(IFracUpdatedListener iful){
        miUpdateListener=iful;
    }
    public int getNMax() {
        return mNMax;
    }

    public int getHeight() {
        return mnHeight;
    }

    public int getWidth() {
        return mnWidth;
    }

    public double getCenterX() {
        return mdCenterX;
    }

    public double getCenterY() {
        return mdCenterY;
    }

    public double x2FracVal(int nX) {
        return mdCenterX + (-(double)mnWidth/2+((double)nX)) / ((double)mnZoom);
    }

    public double y2FracVal(int nY) {
        return mdCenterY + ((double)mnHeight/2-((double)nY)) / ((double)mnZoom);
    }

    public int getZoom() {
        return mnZoom;
    }

    public void setNMax(int nmax) {
        mNMax = nmax;
        if (miChangeListener!=null) miChangeListener.fractalChanged();
        DFracLog.log(3,"Fractal: setNMax:");
    }

    public void setBoundaries(int nWidth, int nHeight) throws EDFracError {
        mnWidth = nWidth;
        mnHeight = nHeight;
        if (miChangeListener!=null) miChangeListener.fractalChanged();
        DFracLog.log(3,"Fractal: setBounds:");
    }

    public void calculate() throws EDFracError{
    	if (miChangeListener!=null) miChangeListener.fractalChanged();
        DFracLog.log(3,"Fractal: calc:");
    }


    public void changeSelection(int nCenter_x, int nCenter_y, double dZoomfactor)
    throws EDFracError {
        if(nCenter_x<0 || nCenter_x>mnWidth ||
        nCenter_y<0 || nCenter_y>mnHeight )
            throw new EDFracError("Center Index out of bound!");
        else{
            mdCenterX+= (-(double)mnWidth/2+((double)nCenter_x)) / ((double)mnZoom);
            mdCenterY+= ((double)mnHeight/2-((double)nCenter_y)) / ((double)mnZoom);
        }
        DFracLog.log(3,"Fractal: change Selection: New Center: x="+mdCenterX+"  y="+mdCenterY);

        // update zoom AFTER changing the center!
        mnZoom = (int)((double)mnZoom*dZoomfactor);
       if (miChangeListener!=null) miChangeListener.fractalChanged();

    }

    public void setCenter(double dNewCenterX, double dNewCenterY)
    throws EDFracError {
        if(dNewCenterX<-2 || dNewCenterX>2 ||
        dNewCenterY<-2 || dNewCenterY>2 )
            throw new EDFracError("Center Index out of bound!");
        mdCenterX=dNewCenterX;
        mdCenterY=dNewCenterY;
        DFracLog.log(3,"Fractal: setCenter:");
       if (miChangeListener!=null) miChangeListener.fractalChanged();

    }

    public void setZoom(int nNewZoom) throws EDFracError {
        if(nNewZoom<0) throw new EDFracError("Negative Zoom not allowed!");
        mnZoom = nNewZoom;
        if (miChangeListener!=null) miChangeListener.fractalChanged();
        DFracLog.log(3,"Fractal: setZoom:");
    }

    public void initFracPieces(int nPieces) {
        DFracLog.log(3,"Fractal:initPieces: "+nPieces+" Pieces initialized.");
        maFracPieces = new ClFracPiece[nPieces];
        int nPiecesX,nPiecesY;
        nPiecesX = nPiecesY = Math.round((float)Math.sqrt(nPieces));
        int nHPP = (int)Math.ceil(mnHeight/nPiecesY);
        int nWPP = (int)Math.ceil(mnWidth/nPiecesX);

        for(int i=0; i<nPieces; i++){
            maFracPieces[i]=new ClFracPiece(i,nWPP*(i%nPiecesX),nHPP*(i/nPiecesY),nWPP,nHPP,
    		mnZoom,mNMax,mnWidth,mnHeight,mdCenterX,mdCenterY);
        }
    }

    public IFracPiece[] getFracPieces() {
        return maFracPieces;
    }

    public void reset() {
        mnWidth=0;
        mnHeight=0;
        mdCenterX=0.0;
        mdCenterY=0.0;
        mnZoom=100;
        if (miChangeListener!=null) miChangeListener.fractalChanged();
    }

    public void setFracPiece(IFracPiece ifp) {
        maFracPieces[ifp.getNumber()]=(ClFracPiece)ifp;
        if (miUpdateListener!=null) miUpdateListener.pointsUpdated();
        DFracLog.log(3,"Fractal: setted "+ifp);
    }

    public void selectionChanged(int center_x, int center_y, double zoomfactor){
        try{ this.changeSelection(center_x, center_y, zoomfactor); }
        catch(Exception e){return;}
        DFracLog.log(3,"Fractal: Selection changed! New Center: x="+mdCenterX+"  y="+mdCenterY);
    }

    public void distributedCalc() {
        if (miChangeListener!=null) miChangeListener.fractalChanged();
        DFracLog.log(3,"Fractal: distcalc called");
    }
    public String toString(){
        String s = "Fractal{\n"
            +"\t#Pieces: "+this.maFracPieces.length  + "\n"
            +"\tWidth:  "+this.mnWidth              + "\n"
            +"\tHeight: "+this.mnHeight             + "\n"
            +"\tCenterX:"+this.mdCenterX            + "\n"
            +"\tCenterY:"+this.mdCenterY            + "\n"
            +"\tZoom:   "+this.mnZoom               + "\n"
            +"\tN-Max:  "+this.mNMax                + "\n";
        if (DFracLog.LogLevel>4){
            s += "\tPieces:{\n";
            for (int i=0;i<maFracPieces.length;i++)
                s += maFracPieces[i].longString("\t\t",false) + "\n";
            s += "\t}\n";
        }
        s += "}";

        return s;
    }


}

