/** Class that represents a Fractal Piece as defined by IFracPiece
 *  is serializable to be sent over the net as a piece of frac (code does not get serialized)
 *
 */

package DAL;

import util.*;
import DAL.*;
import java.util.Vector;
import java.util.Iterator;
import java.io.*;

public class ClFracPiece extends Thread implements Serializable,IFracPiece{
    //all fields get serialized 2B sent over net
    private int mnNumber;
    private int[] maPoints=null;
    private int mnX;
    private int mnY;
    private int mnWidth;
    private int mnHeight;
    
    private int mnTotalWidth;
    private int mnTotalHeight;
    private double mdCenterX;
    private double mdCenterY;
    private int mnZoom;
    private int mnNMax;
    
    public volatile boolean bIsBusy=false;
    private volatile byte mnProgress=0;
    public volatile byte mnMode;
    
    
    //fields
    
    //private ICalcPerformedListener moCalcPerformedListener=null;
    
    public void setPoints(int[] moarray){
        maPoints=moarray;
        mnProgress=100;
        bIsBusy=false;
        mnMode=STOP;
        DFracLog.log(3,"FractalPiece: setPoints");
    }
    
    public ClFracPiece(){
        mnMode=0;
    }
    
    public ClFracPiece(byte mode){
        mnMode=mode;
    }
    public ClFracPiece(byte mode,int major, int minor){
        mnMode=mode;
        mnX=major;
        mnY=minor;
    }
    public ClFracPiece(int num, int x, int y, int width, int height,
    int zoom, int nMax,int totX,int totY,double centerX,double centerY){
        //moCalcPerformedListener=oICPL;
        mnNumber = num;
        mnWidth=width;
        mnHeight=height;
        mnTotalWidth=totX;
        mnTotalHeight=totY;
        mdCenterX=centerX;
        mdCenterY=centerY;
        
        mnX=x;
        mnY=y;
        mnZoom=zoom;
        mnNMax=nMax;
        mnMode=CALC;
        mnProgress=0;
    }
    //Accessor routines, not serialized...
    public int getX(){
        return mnX;
    }
    public int getY(){
        return mnY;
    }
    public int getHeight(){
        return mnHeight;
    }
    public int getWidth(){
        return mnWidth;
    }
    public int[] getPoints(){
        return maPoints;
    }
    
    public void calculate() {
        if (mnProgress<100 && !bIsBusy){
            DFracLog.log(3,"FracPiece: "+this+" : Calculating... ");
            maPoints = new int[mnWidth*mnHeight];
            bIsBusy=true;
            this.mnMode=CALC;
            
            int index=0;
            for(int yb=0; yb<mnHeight; ++yb){
                double b = ((double)mnTotalHeight/2-((double)(mnY+yb))) / ((double)mnZoom) + mdCenterY;
                for(int xa=0;xa<mnWidth; ++xa){
                    double a = (-(double)mnTotalWidth/2+((double)(mnX+xa))) / ((double)mnZoom) + mdCenterX;
                    maPoints[index++] = calcMandel(a,b);
                    mnProgress = (byte)Math.round(((float)index/(float)maPoints.length)*100);
                    
                }
            }
            DFracLog.log(3,"FracPiece "+this+" : all calculated ("+mnProgress+"%)");
            bIsBusy=false;
            mnMode=IFracPiece.STOP;
        }
    }
    
    public int getNumber(){
        return this.mnNumber;
    }
    public double getFracCenterX() {
        return this.mdCenterX;
    }
    
    public double getFracCenterY() {
        return this.mdCenterY;
    }
    
    public int getFracHeight() {
        return this.mnTotalHeight;
    }
    
    public int getFracWidth() {
        return this.mnTotalWidth;
    }
    
    public int getNMax() {
        return this.mnNMax;
    }
    
    public int getZoom() {
        return this.mnZoom;
    }
    
    public boolean isReady(){
        return (mnProgress==100 && !bIsBusy);
    }
    //end accessor
    
    public void run(){
        try{
            this.calculate();
        } catch(Exception ex){
            DFracLog.log(1,"FracPiece "+this+" Interrupted: "+ex.getMessage());
            bIsBusy=false;
        }
        
    }
    
    private int calcMandel(double a, double b){
        double tmp, x=0, y=0;
        int i=0;
        do {
            ++i;
            tmp  = x*x - y*y + a;
            y = 2*x*y + b;
            x = tmp;
        } while (((x*x + y*y) <= 4) && (i < mnNMax));
        return i;
    }
    
    
    public String toString(){
        if (DFracLog.LogLevel>4) return longString("",true);
        else return new String("Piece "+mnNumber+":x["+mnX+"-"+(mnX+mnWidth)+"] y["
        +mnY+"-"+(mnY+mnHeight)+"]:"+mnProgress+"%");
    }
    public String longString(String ind, boolean bShowPoints){
        String sp="";
        if (bShowPoints){
            sp += ind + "\tsome Points{\n";
            if (this.maPoints==null) sp += ind + "\t\tno Points!";
            else{
                sp += ind + "\t\t";
                sp += maPoints[0];
                for(int i=1; i<maPoints.length/100;++i){
                    sp += " .. " + maPoints[i];
                    if (i%20==0) sp += ("\n" + ind +"\t\t");
                }
                sp += "\n";
            }
            sp += ind + ("\t}\n");
        }
        String s = ind + "FractalPiece{\n"
        + ind + "\tNumber:       "+this.mnNumber       + "\n"
        + ind + "\tX-Pos:        "+this.mnX            + "\n"
        + ind + "\tY-Pos:        "+this.mnY            + "\n"
        + ind + "\tWidth:        "+this.mnWidth        + "\n"
        + ind + "\tHeight:       "+this.mnHeight       + "\n"
        + ind + "\tFractalWidth: "+this.mnTotalWidth   + "\n"
        + ind + "\tFractalHeight:"+this.mnTotalHeight  + "\n"
        + ind + "\tCenterX:      "+this.mdCenterX      + "\n"
        + ind + "\tCenterY:      "+this.mdCenterY      + "\n"
        + ind + "\tZoom:         "+this.mnZoom         + "\n"
        + ind + "\tN-Max:        "+this.mnNMax         + "\n"
        + ind + "\tProgress:     "+this.mnProgress     + "\n"
        +sp
        + ind + "}";
        return s;
    }
    
    public void update(int num, int x, int y, int width, int height, int zoom, int nMax, int totX, int totY, double centerX, double centerY) {
        
        this.mnProgress=0;
        mnNumber = num;
        mnWidth=width;
        mnHeight=height;
        mnTotalWidth=totX;
        mnTotalHeight=totY;
        mdCenterX=centerX;
        mdCenterY=centerY;
        
        mnX=x;
        mnY=y;
        mnZoom=zoom;
        mnNMax=nMax;
        mnMode=CALC;
    }
    
    public byte getProgress() { return mnProgress; }
    
    
}
