/** Base PL class, implementing IPL and offering a PL with all its listeners
 *
 */

package PL;

import DAL.*;

import java.util.*;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.color.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.beans.*;
import util.*;

public class ClDFracWidget extends JPanel implements IPL,IFracUpdatedListener{
    
    private static int mnWIDTH = 0;
    private static int mnHEIGHT = 0;
    //private static long mlSMOOTHPAINTDELAY = 300;
    
    /** Stack for Frac Pieces waiting to be painted */
    private Stack moUpdatedFracPieces=null;
    /** DAL of the diagram */
    private IFrac moFrac=null;
    /** Image Buffer for the Frac */
    private BufferedImage moFracImgBuf=null;
    /** Graphics of the buffered images */
    private Graphics2D moG2d_buf=null;
    /** Timer for smooth drawing */
    //private ClTimer moSmoothDrawTimer=null;
    /** Mark Color changes */
    private boolean mbColorChanged=false;
    /** field for status messages */
    private JTextField moTxtField;
    /** captures the mouse position */
    private Point moPoint=null;
    /** captures the mouse click start position */
    private Point moStartPoint=null;
    /** boolean for mouse drag */
    private boolean mbDrag = false;
    /** captures the approximate matrix coordinates of actual mouse position */
    private Point moMatPos = null;
    /** Listener for selection-change callbacks */
    private ISelChangedListener moPaneChangeListener;
    /** The used color effect */
    private byte mColorEffect=0;
    /** The number formatter (why do we need this? )*/
    NumberFormat moNF = NumberFormat.getInstance(Locale.US);
    
    
    
    /** Constructor for new PL with DAL backend and status bar
     * @param poDiagram The DAL backend of this diagram
     * @param poTextField Field for status messages
     */
    public ClDFracWidget(JTextField oTextField,Stack readyPieces) {
        moUpdatedFracPieces = readyPieces;
        moTxtField = oTextField;
        moStartPoint = new Point();
        moPoint = new Point();
        //moUpdatedFracPieces = new Stack();
        //moSmoothDrawTimer=new ClTimer();
        moNF.setMaximumFractionDigits(4); // fix output
        moNF.setMinimumFractionDigits(4);
        moNF.setMaximumIntegerDigits(1);
        moNF.setMinimumIntegerDigits(1);
        
        
        addComponentListener(
        new ComponentListener(){
            public void componentHidden(ComponentEvent e){}
            public void componentMoved(ComponentEvent e){}
            public void componentResized(ComponentEvent e) {
                Component c = e.getComponent();
                mnWIDTH=c.getSize().width;
                mnHEIGHT=c.getSize().height;
                if(moFrac!=null){
                    try{
                        if( moFrac.getHeight()!=mnHEIGHT &&
                        moFrac.getWidth()!=mnWIDTH ) {
                            moFrac.setBoundaries(mnWIDTH, mnHEIGHT);
                        }
                    }catch(EDFracError err){}
                }
                moFracImgBuf = new BufferedImage(mnWIDTH,
                mnHEIGHT,
                BufferedImage.TYPE_INT_RGB);
                moG2d_buf = moFracImgBuf.createGraphics();
                if(moPaneChangeListener!=null){
                    moPaneChangeListener.distributedCalc();
                }
            }
            public void componentShown(ComponentEvent e){}
        }
        );
        
        addMouseListener(
        new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                repaint();
            }
            
            public void mousePressed(MouseEvent e) {
                moStartPoint.x = e.getX();
                moStartPoint.y = e.getY();
                mbDrag = true;
            }
            
            public void mouseReleased(MouseEvent e) {
                mbDrag = false;
                repaint();
                Point oCenter = getSelectionCenter();
                moPaneChangeListener.selectionChanged(oCenter.x,
                oCenter.y,
                getZoomFactor());
            }
        }
        );
        addMouseMotionListener(
        new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent e){
                moMatPos = new Point(e.getX(), e.getY());
                moTxtField.setText("["
                +moNF.format(moFrac.x2FracVal(moMatPos.x))+"]["
                +moNF.format(moFrac.y2FracVal(moMatPos.y))
                +"]");
            }
            
            public void mouseDragged(MouseEvent e) {
                moPoint.x = e.getX();
                moPoint.y = e.getY();
                moMatPos = new Point(e.getX(), e.getY());
                Rectangle r = getSelection();
                moTxtField.setText("["
                +moNF.format(moFrac.x2FracVal(moStartPoint.x))+"]["
                +moNF.format(moFrac.y2FracVal(moStartPoint.y))+"]"
                +"->["
                +moNF.format(moFrac.x2FracVal(r.x+r.width))+"]["
                +moNF.format(moFrac.y2FracVal(r.y+r.height))
                +"]");
                repaint();
            }
        }
        );
    }
    
    
    /** The must have paint method that implements jframe's paint method (
     *  this is called for each repaint())
     * @param g Graphics component, used to paint things
     */
    public void paintComponent(Graphics g) {
        my_repaint(g);
    }
    
    public synchronized void my_repaint(Graphics g) {
        Graphics2D g2d = (Graphics2D)g;
        
        // paint Fractal
        if((moUpdatedFracPieces.size()!=0) || (moFracImgBuf==null)){
            DFracLog.log(3,"DFracWidget:repaint: Updating Fractal,"+moUpdatedFracPieces.size()
            +" new Pieces");
            while(moUpdatedFracPieces.size()!=0){
                IFracPiece oUpdatedFracPiece=null;
                synchronized(moUpdatedFracPieces){
                    oUpdatedFracPiece = (IFracPiece)moUpdatedFracPieces.pop();
                }
                paintFracPiece(moG2d_buf, oUpdatedFracPiece);
            }
        }
        else if(mbColorChanged){
            IFracPiece[] oaFracPieces=moFrac.getFracPieces();
            for (int i=0; i<oaFracPieces.length; ++i){
                paintFracPiece(moG2d_buf, oaFracPieces[i]);
            }
            mbColorChanged=false;
        }
        // update buffered fractal image
        g2d.drawImage(moFracImgBuf, 0, 0, this);
        
        // paint zoom-selection-box
        if(mbDrag){
            Rectangle r = getSelection();
            g2d.setColor(Color.GREEN);
            //g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, 0.5f));
            //setColor(new Color(Color.BITMASK));
            g2d.drawRect(r.x, r.y, r.width, r.height);
            g2d.drawOval(r.x+r.width/2, r.y+r.height/2, 3,3);
            g2d.setColor(Color.MAGENTA);
            g2d.drawRect(r.x+1, r.y+1, r.width-2, r.height-2);
            g2d.drawOval(r.x+1+r.width/2, r.y+1+r.height/2, 1,1);
        }
    }
    
    
    /** If any changes in DAL are made (e.g. new DAL), you can set/change it
     *  this way, the diagram will be repainted
     * @param oFractal The DAL to use
     */
    public void setDALObj(IFrac oFractal) {
        moFrac = oFractal;
        repaint();
    }
    
    /** If you want to be informed about any changes in selection, you can get a
     *  callback by offering an ISelChangedListener
     * @param poSelListener Listener for selection-information callbacks
     */
    public void setSelChangedListener(ISelChangedListener poSelListener) {
        moPaneChangeListener = poSelListener;
    }
    
    public void paintFracPiece(Graphics2D g2d, IFracPiece oFracPiece){
        try{
            if(oFracPiece!=null){
                DFracLog.log(4,"DFracWidget:repaint:paintFracPiece: "+oFracPiece);
                int w = oFracPiece.getWidth();
                int h = oFracPiece.getHeight();
                int nXPos = oFracPiece.getX();
                int nYPos = oFracPiece.getY();
                g2d.setColor(Color.BLACK);
                g2d.fillRect(nXPos, nYPos, w, h);
                if(oFracPiece.isReady()){
                    int[] a=oFracPiece.getPoints();
                    int index=0;
                    int pic[] = new int[w*h];
                    // we do not work on original data here, because that would
                    // make change of color effects impossible. But the pic array
                    // won't make much overhead, when the pieces are small enough.
                    int nMaxDepth = moFrac.getNMax();
                    for (int x=0; x<w; ++x) {
                        for (int y=0; y<h; ++y) {
                            if(a[index]==nMaxDepth) pic[index]=0;
                            else{
                                switch(mColorEffect){
                                    case IPL.COLOREFF0:
                                        pic[index] = smoothColorRGB(a[index]%256);
                                        break;
                                    case IPL.COLOREFF1:
                                        pic[index] = smoothColorHSV(a[index]%256);
                                        break;
                                    case IPL.COLOREFF2:
                                        pic[index] = snoopyColor(a[index]%256);
                                        break;
                                    case IPL.COLOREFF3:
                                        pic[index] = gainColored(a[index]%256);
                                        break;
                                    case IPL.COLOREFF4:
                                        pic[index] = smoothBlue2Black(a[index]%256);
                                        break;
                                    default:
                                        pic[index] = smoothColorRGB(a[index]%256);
                                }
                            }
                            ++index;
                        }
                    }
                    Image img = createImage(
                    new MemoryImageSource(w, h, IndexColorModel.getRGBdefault(),
                    pic, 0, w));
                    g2d.drawImage(img, nXPos, nYPos,w , h, this);
                }
            }
        }catch(Exception e){
            DFracLog.log(1,"DFracWidget:repaint:paintFracPiece: " + ( (oFracPiece!=null) ? oFracPiece.toString() : "") );
        }
    }
    
    
    private double getRatio(){
        return ((double)mnWIDTH)/((double)mnHEIGHT);
    }
    
    private Rectangle getSelection(){
        Rectangle r = new Rectangle();
        r.width = Math.abs(moStartPoint.x - moPoint.x);
        r.height = Math.abs(moStartPoint.y - moPoint.y);
        
        double dRatio = ((double)r.width)/((double)r.height);
        if(dRatio>=getRatio()) r.height = (int)(((double)r.width)/getRatio());
        else r.width =(int)(((double)r.height)*getRatio());
        
        if(moStartPoint.x<=moPoint.x) r.x=moStartPoint.x;
        else r.x=moStartPoint.x-r.width;
        
        if(moStartPoint.y<=moPoint.y) r.y=moStartPoint.y;
        else r.y=moStartPoint.y-r.height;
        
        return r;
    }
    
    
    private Point getSelectionCenter(){
        Rectangle r = getSelection();
        Point center = new Point();
        center.x = r.x + r.width/2;
        center.y = r.y + r.height/2;
        return center;
    }
    
    private double getZoomFactor(){
        Rectangle r = getSelection();
        return ((double)mnWIDTH)/((double)r.width);
    }
    
    
    private int smoothColorRGB(int i){
        int r = 0, g = 0, b = 0;
        r=20*i/255;//(int)(30.0*((double)i)/255.0);
        g=150-150*i/255;
        b=255-i;//100-(int)(100.0*((double)i)/255.0);
        return (0xff<<24 | r<<16 | g<<8 | b);
    }
    
    private int smoothColorHSV(int i){
        return Color.HSBtoRGB((float)((306+i)/(double)255), (float)1, (float)1);
    }
    
    private int smoothBlue2Black(int i){
        // blue to black, many colors
        return (0xff<<24 | (32+i*3)<<16 | (32+i)<<8 | (255-i));
    }
    
    private int snoopyColor(int i) {
        int h = 0, s = 0, v = 0;
        // from red to yellow
        if(i>90){      h=(i-90)*25;    s=255;       v=i;}  // 90 entscheidet Apfelmannfarbe
        else if(i>80){ h=255-(i-40)*5; s=230;       v=i+70;}
        else if(i>70){ h=215-(i-70)*3; s=255;       v=90+(i-45)*4;}
        else if(i>60){ h=255-(i-20)*5; s=255;       v=i+170;}
        else if(i>50){ h=255-(i-20)*5; s=250;       v=120+(i-50)*12;}
        else if(i>40){ h=255-(i-10)*6; s=200;       v=255;}
        else if(i>30){ h=255-(i-10)*8; s=200;       v=200;}
        else if(i>20){ h=145+i*2;      s=220;       v=205-(i-20)*6;} // blau verlauf
        else if(i>10){ h=145+i*2;      s=220;       v=255-(i-10)*5;} // blau verlauf
        else if(i>0){  h=200-i*20;     s=255;       v=160+i;} // Regenbogen
        else {         h=0;            s=0;         v=0;}
        
        return Color.HSBtoRGB(((float)h)/255, ((float)s)/255, ((float)v)/255);
    }
    
    private int gainColored(int i) {
        int r = 0, g = 0, b = 0;
        // from red to yellow
        if (i < 40) { r = 255; g = 15+(i%40)*6; b = 15; }
        // from yellow to green
        else if (i < 80) { r = 255-(i%40)*6; g = 255; b = 15; }
        // from green to light-blue
        else if (i < 120) { r = 15; g = 255; b = 15+(i%40)*6; }
        // from light-blue to blue
        else if (i < 160) { r = 15; g = 255-(i%40)*6; b = 255; }
        // from blue to pink
        else if (i < 200) { r = 15+(i%40)*6; g = 15; b = 255; }
        // from pink to red
        else if (i < 240) { r = 255; g = 15; b = 255-(i%40)*6; }
        return (0xff<<24 | r<<16 | g<<8 | b);
    }
    
    public void changeColorEffect(byte colEffect) {
        mColorEffect=colEffect;
        mbColorChanged=true;
        repaint();
    }
    
    public BufferedImage getImage(){
        return moFracImgBuf;
    }
    
    public void pointsUpdated() {
        repaint();
    }
    public void forcedRepaint(){
        this.my_repaint(getGraphics());
    }
    
    
}
