#include <stdio.h>
#include <orb/orbit.h>

#include "DFracCorba.h"

static gboolean raised_exception(CORBA_Environment *ev) {
	return ((ev)->_major != CORBA_NO_EXCEPTION);
}

static void abort_if_exception(CORBA_Environment *ev, const char* mesg) {
	if (raised_exception (ev)) {
		g_error ("%s %s", mesg, CORBA_exception_id (ev));
		CORBA_exception_free (ev); 
		abort(); 
	}
}

int main (int argc, char *argv[]){
	FILE * ifp;
	char * ior, filebuffer[1024];
	DFracCorba_doubleArray *dArrFrac;
	DFracCorba_longArray *nArrFrac;
	CORBA_Environment ev[1];
	CORBA_ORB orb;            /* ORB */
	DFracCorba_DFracCorba dfraccorba_client;         /* the service */
	int i,j,index=0;

	dArrFrac = CORBA_sequence_CORBA_double__alloc ();
	dArrFrac->_buffer = CORBA_sequence_CORBA_double_allocbuf (10);
	dArrFrac->_length = 10;
	dArrFrac->_buffer[0] = 80.0; // FracWidth
        dArrFrac->_buffer[1] = 22.0; // FracHeight
        dArrFrac->_buffer[2] = 0.0; // CenterX
        dArrFrac->_buffer[3] = 0.0; // CenterY
        dArrFrac->_buffer[4] = 15.0; // Zoom
        dArrFrac->_buffer[5] = 500.0; // NMax
        dArrFrac->_buffer[6] = 0.0; // FracPiece upper left corner X
        dArrFrac->_buffer[7] = 0.0; // FracPiece upper left corner Y
        dArrFrac->_buffer[8] = 80.0; // FracPiece Width
        dArrFrac->_buffer[9] = 22.0; // FracPiece Height
	
	CORBA_exception_init(ev);
	orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", ev);
	abort_if_exception(ev, "init ORB failed");

	ifp = fopen("dfrac.ior","r");
	if( ifp == NULL ) {
		g_error("can not open \"dfrac.ior\"");
		abort ();
	}
	fgets(filebuffer,1023,ifp);
	ior = g_strdup(filebuffer);
	fclose(ifp);

	dfraccorba_client = CORBA_ORB_string_to_object(orb, ior, ev);
	abort_if_exception(ev, "bind failed");

	g_print("Got object. Starting calculation.\n");
	nArrFrac=DFracCorba_DFracCorba_calculate(dfraccorba_client,dArrFrac,ev);
	abort_if_exception(ev, "service not reachable");
      
	for(i=0; i<22;++i){
		for(j=0; j<80;++j){
			if(nArrFrac->_buffer[index]<10) printf(" ");
			else if(nArrFrac->_buffer[index]<20) printf(".");
			else if(nArrFrac->_buffer[index]<30) printf(".");
			else if(nArrFrac->_buffer[index]<40) printf(":");
			else if(nArrFrac->_buffer[index]<50) printf(":");
			else if(nArrFrac->_buffer[index]<60) printf("+");
			else if(nArrFrac->_buffer[index]<70) printf("+");
			else if(nArrFrac->_buffer[index]<80) printf("x");
			else if(nArrFrac->_buffer[index]<90) printf("x");
			else if(nArrFrac->_buffer[index]<100) printf("#");
			else printf("#");
			++index;
		}
		printf("\n");
	}
	
	/* Clean up */
	CORBA_Object_release(dfraccorba_client, ev);
	abort_if_exception(ev, "releasing service failed");

	CORBA_ORB_destroy (orb, ev);
	abort_if_exception(ev, "cleanup failed");
    
	/* successfull termination */
	exit (0);
}
