#include "DFracCorba.h"

/*** App-specific servant structures ***/

typedef struct
{
   POA_DFracCorba_DFracCorba servant;
   PortableServer_POA poa;

}
impl_POA_DFracCorba_DFracCorba;

/*** Implementation stub prototypes ***/

static void impl_DFracCorba_DFracCorba__destroy(impl_POA_DFracCorba_DFracCorba
						* servant,
						CORBA_Environment * ev);
static DFracCorba_longArray
   *impl_DFracCorba_DFracCorba_calculate(impl_POA_DFracCorba_DFracCorba *
					 servant,
					 DFracCorba_doubleArray *
					 fracpieceData,
					 CORBA_Environment * ev);

/*** epv structures ***/

static PortableServer_ServantBase__epv impl_DFracCorba_DFracCorba_base_epv = {
   NULL,			/* _private data */
   NULL,			/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_DFracCorba_DFracCorba__epv impl_DFracCorba_DFracCorba_epv = {
   NULL,			/* _private */
   (gpointer) & impl_DFracCorba_DFracCorba_calculate,

};

/*** vepv structures ***/

static POA_DFracCorba_DFracCorba__vepv impl_DFracCorba_DFracCorba_vepv = {
   &impl_DFracCorba_DFracCorba_base_epv,
   &impl_DFracCorba_DFracCorba_epv,
};

/*** Stub implementations ***/

static DFracCorba_DFracCorba
impl_DFracCorba_DFracCorba__create(PortableServer_POA poa,
				   CORBA_Environment * ev)
{
   DFracCorba_DFracCorba retval;
   impl_POA_DFracCorba_DFracCorba *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_DFracCorba_DFracCorba, 1);
   newservant->servant.vepv = &impl_DFracCorba_DFracCorba_vepv;
   newservant->poa = poa;
   POA_DFracCorba_DFracCorba__init((PortableServer_Servant) newservant, ev);
   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_DFracCorba_DFracCorba__destroy(impl_POA_DFracCorba_DFracCorba * servant,
				    CORBA_Environment * ev)
{
   PortableServer_ObjectId *objid;

   objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
   PortableServer_POA_deactivate_object(servant->poa, objid, ev);
   CORBA_free(objid);

   POA_DFracCorba_DFracCorba__fini((PortableServer_Servant) servant, ev);
   g_free(servant);
}

static DFracCorba_longArray *
impl_DFracCorba_DFracCorba_calculate(impl_POA_DFracCorba_DFracCorba * servant,
				     DFracCorba_doubleArray * fracpieceData,
				     CORBA_Environment * ev)
{
   DFracCorba_longArray *fracpieceCalc;
   int nFracWidth = (int)fracpieceData->_buffer[0];
   int nFracHeight = (int)fracpieceData->_buffer[1];
   double dCenterX = fracpieceData->_buffer[2];
   double dCenterY = fracpieceData->_buffer[3];
   int nZoom = (int)fracpieceData->_buffer[4];
   int nMax = (int)fracpieceData->_buffer[5];
   int nFracPieceULCornerX = (int)fracpieceData->_buffer[6];
   int nFracPieceULCornerY = (int)fracpieceData->_buffer[7];
   int nFracPieceWidth = (int)fracpieceData->_buffer[8];
   int nFracPieceHeight = (int)fracpieceData->_buffer[9];
   fracpieceCalc = CORBA_sequence_CORBA_long__alloc ();

   fracpieceCalc->_buffer = CORBA_sequence_CORBA_long_allocbuf (nFracPieceWidth*nFracPieceHeight);
   fracpieceCalc->_length = nFracPieceWidth*nFracPieceHeight;
   /* ------   insert method code here   ------ */
   calculateFracPiece(
	fracpieceCalc,
	nFracWidth,
	nFracHeight,
	dCenterX,
	dCenterY,
	nZoom,
	nMax,
	nFracPieceULCornerX,
	nFracPieceULCornerY,
	nFracPieceWidth,
	nFracPieceHeight );
   /* ------ ---------- end ------------ ------ */

   return fracpieceCalc;
}

static void calculateFracPiece(
		DFracCorba_longArray *fracpieceCalc,
		int nFracWidth,
		int nFracHeight,
		double dCenterX,
		double dCenterY,
		int nZoom,
		int nMax,
		int nFracPieceULCornerX,
		int nFracPieceULCornerY,
		int nFracPieceWidth,
		int nFracPieceHeight ){
        int index=0, yb, xa;
	double a, b;
	for(yb=0; yb<nFracPieceHeight; ++yb){
            b = ((double)nFracHeight/2-((double)(nFracPieceULCornerY+yb))) / ((double)nZoom) + dCenterY;
	     for(xa=0;xa<nFracPieceWidth; ++xa){
                a = (-(double)nFracWidth/2+((double)(nFracPieceULCornerX+xa))) / ((double)nZoom) + dCenterX;
                fracpieceCalc->_buffer[index++] = (unsigned long)mandelcalculation(a,b,nMax);
	    }
        }
}

static int mandelcalculation(double a, double b, int nMax){
        double tmp, x=0, y=0;
        int i=0;
        do {
            ++i;
            tmp  = x*x - y*y + a;
            y = 2*x*y + b;
            x = tmp;
        } while (((x*x + y*y) <= 4) && (i < nMax));
        return i;
}

