/*
 * IWorkerThread.java
 *
 * Created on 13. Mai 2004, 14:44
 */

package net;
import Control.*;
import DAL.*;
import util.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *
 * @author  Vitus
 */
public abstract class ClAbstractWorkerThread extends Thread{
    
    protected volatile boolean bRunning;
    //protected volatile boolean bNew;
    protected volatile IFracPiece ifp;
    protected JLabel moStatusLabel;
    protected JPanel myPanel;
    public volatile int mnIterations;
    protected volatile byte ms8Progress;
    protected ClAbstractCalcProvider moAcp;
    protected String msServer;
    protected volatile boolean bAlreadyChecking=false;
    protected volatile boolean mbConnected=false;
    public String msVersionString;
    
    public synchronized int incIterations(){
        return ++mnIterations;
    }
    
    public void init(JPanel mpanel, JLabel mlabel, String name,ClAbstractCalcProvider acp){
        this.moAcp = acp;
        this.myPanel = mpanel;
        this.moStatusLabel = mlabel;
        super.setName(name);
        DFracLog.log(3,getName()+":initialized");
        
    }
    
    public void beginCalcing() {
        if (!this.mbConnected) return;
        this.bRunning=true;
        synchronized(this){ notify(); }
    }
    
    public boolean isRunning(){ return bRunning; }
    public boolean isReady(){ return (ifp.isReady() && ms8Progress==100); }
    
    public IFracPiece getPiece(){
        return ifp;
    }
    
    public int getProgress(){
        byte t=0;
        try{ t=ifp.getProgress(); }
        catch(Exception e){ DFracLog.log(2,getName()+":calcing: Error obtaining Progress:"+e);}
        return  ms8Progress=((ms8Progress > t)?ms8Progress:t) ;
    }
    
    public void run(){
        DFracLog.log(3,getName()+":running");
        
        try{
            while (true){
                synchronized(this){
                    wait();
                }
                try{
                    while(bRunning && mbConnected){
                        if (null==(ifp=this.moAcp.getNewPiece())) Thread.sleep(100);
                        else{
                            DFracLog.log(3,getName()+"setNewPiece: "+ifp);
                            setNewPiece(ifp);
                            if (!ifp.isReady()){
                                moStatusLabel.setText("[piece"+ifp.getNumber()+"]");
                                DFracLog.log(3,getName()+":calcing: "+ifp);
                                if (calculate()){
                                    ms8Progress=100;
                                    moStatusLabel.setText("[v"+msVersionString+" ready]");
                                    DFracLog.log(3,getName()+":calculated: "+ifp);
                                    moAcp.calcSucceeded(ifp);
                                }else{
                                    DFracLog.log(1,getName()+":calcing: rescheduling "+ifp);
                                    this.moAcp.reSchedulePieceAndCheck(this);
                                }
                            }
                        }
                        
                    }
                    
                    
                }catch(Exception e){
                    DFracLog.log(1,getName()+":calcing: Exception:"+e);
                    DFracLog.log(1,getName()+":calcing: rescheduling "+ifp);
                    moStatusLabel.setText("[error]");
                    this.moAcp.reSchedulePieceAndCheck(this);
                }
                
            }
        }catch(InterruptedException e){return;}
    }
    private void setNewPiece(IFracPiece ifps){
        ifp=ifps;
        synchronized(this){
            mnIterations = ms8Progress = 0;
        }
        
    }
    public void updateProgress(){
        try{
            DFracLog.log(4,getName()+": ProgressUpdate:"+ms8Progress+"%");
            moStatusLabel.setText("[piece"+ifp.getNumber()+"]:"+ms8Progress+"%");
            synchronized(this){
                this.mnIterations=0;
            }
        }catch(Exception e){}
    }
    
    public boolean calculate(){
        return this.mbConnected;
    }
    public boolean reConnect(){
        if (this.mbConnected){ end(); disConnect(); }
        return connect(this.msServer);
    }
    
    public boolean checkServer(){
        if (!this.mbConnected) return false;
        else if (this.bAlreadyChecking) return true;
        this.bAlreadyChecking = true;
        if (this.mnIterations > ICalcProvider.IMPLICIT_TIMEOUT){
            DFracLog.log(1,getName()+":checkServer:Implicit Timeout hit!");
            return false;
        }
        return true;
    }
    
    public boolean connect(String server){
        this.msServer = server;
        this.mbConnected = true;
        return true;
    }
    public void disConnect(){
        this.mbConnected=false;
    }
    public void end(){ bRunning = false; }
    
    
}
