/*
 * ClCorbaStub.java
 *
 * Created on 14. Mai 2004, 11:36
 */

package net;

import Control.*;
import DAL.*;
import DFracCorba.*;
import util.*;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextExt.*;


/**
 *
 * @author  Vitus
 */
public class ClCorbaStub extends ClAbstractWorkerThread{
    private volatile double[] daten;
    private CorbaDFracServer dfcserver;
    private volatile int[] fracpieceCalc;
    
    public boolean connect(String server){
        if (mbConnected){
            disConnect();
            DFracLog.log(2,getName()+":connect: already connected");
        }
        super.connect(server);
        String nPort = "1050";
        int t=0;
        
        if ((t=server.indexOf(':'))>0){
            nPort=server.substring(t+1);
            server=server.substring(0,t);
        }
        try{
            String[] servstring={"-ORBInitialPort" ,nPort, "-ORBInitialHost",server};
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(servstring, null);
            
            // get the root naming context
            org.omg.CORBA.Object objRef =
            orb.resolve_initial_references("NameService");
            
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
            // resolve the Object Reference in Naming
            String name = "DFracCalculator";
            dfcserver = CorbaDFracServerHelper.narrow(ncRef.resolve_str(name));
            
            DFracLog.log(3,"Obtained a handle on server object: " + dfcserver);
            
        }catch(Exception e) {
            DFracLog.log(1,getName()+"CorbaError: "+e.getMessage());
            return false;
        }
        return checkServer();
    }
    
    public boolean calculate(){
        if (!this.mbConnected) return false;
        daten = new double[10];
        
        daten[0] = (double)ifp.getFracWidth(); // FracWidth
        daten[1] = (double)ifp.getFracHeight(); // FracHeight
        daten[2] = ifp.getFracCenterX(); // CenterX
        daten[3] = ifp.getFracCenterY(); // CenterY
        daten[4] = (double)ifp.getZoom(); // Zoom
        daten[5] = (double)ifp.getNMax(); // NMax
        daten[6] = (double)ifp.getX(); // FracPiece upper left corner X
        daten[7] = (double)ifp.getY(); // FracPiece upper left corner Y
        daten[8] = (double)ifp.getWidth(); // FracPiece Width
        daten[9] = (double)ifp.getHeight(); // FracPiece Height
        fracpieceCalc = new int[ifp.getFracWidth()*ifp.getFracHeight()];
        String ret = "";
        try{
            ret = dfcserver.calculate(daten);
        }catch(Exception e){
            DFracLog.log(1,getName()+"Error calcing: " + e.getMessage());
            return false;
        }
        try{
            String[] ints = ret.split(",");
            for (int i=0;i<ints.length;i++) fracpieceCalc[i] = Integer.parseInt(ints[i]);
        } catch (Exception e) {
            System.err.println("Error splitting and setting ints:"+e.toString());
            return false;
        }
	
	ifp.setPoints(fracpieceCalc);
        return true;
    }
    
    public boolean checkServer(){
        boolean ret=true;
	if (!super.checkServer()) ret=false;
        try{
            this.msVersionString = dfcserver.getVersion();
        }catch(Exception e){
            DFracLog.log(2,getName()+"checkServer: Error getting Version:" + e.getMessage()+", server dead");
            ret=false;
        }
        if (this.msVersionString.equals("")) ret=false;
        this.bAlreadyChecking = false;
        DFracLog.log(3,getName()+"checkServer:server State: "+ret);
        return (mbConnected = ret);
    }
    
}
