package net;

import util.*;
import DAL.*;

import java.net.*;
import java.io.*;

public class DFracSocketedServer{

    private static final int MAJOR = 0;
    private static final int MINOR = 5;
    
    private ServerSocket moSSocket;
    private volatile ClFracPiece  moFracPiece;
    private ObjectInputStream moOIS;
    private ObjectOutputStream moOOS;
    private Socket moConnSocket;
    private boolean mbConnected=false;
    private int mnPort = 0;

    public DFracSocketedServer(int port){
        System.out.println("DFracSocketedServer v"+MAJOR+"."+MINOR+" starting up...");
        
        if (port==0) mnPort=24567;
        else mnPort=port;
        try{
            this.moSSocket = new ServerSocket(mnPort);
        } catch(Exception e){
            DFracLog.log(1,"DFracSocketedServer: ERROR while creating ServerSocket ");
            System.exit(1);
            return;
        }
        //main program loop
        while(true){

            System.out.println("DFracSocketedServer["+mnPort+"]: waiting for a connection on port "+mnPort);
            try{
                moConnSocket = moSSocket.accept();
                moConnSocket.setKeepAlive(true);
            }catch(Exception e){
                DFracLog.log(1,"DFracSocketedServer["+mnPort+"]: Error accepting connection: "+e.getMessage());
                continue;
            }
            DFracLog.log(1,"DFracSocketedServer["+mnPort+"]: got a connection from "+moConnSocket.getRemoteSocketAddress());

            //build streams:
            try{
                moOOS = new ObjectOutputStream(moConnSocket.getOutputStream());
            }catch(Exception e){
                DFracLog.log(1,"DFracSocketedServer["+mnPort+"]: Error initializing Streams: "+e.getMessage());
                continue;
            }
            mbConnected=true;


            //ready for main connxion loop:
            DFracLog.log(3,"DFracSocketedServer["+mnPort+"]: waiting for data...");

            while(mbConnected){
                //ClFracPiece cfp = moFracPiece;


                if (moConnSocket.isClosed() || moConnSocket.isInputShutdown() ||
                moConnSocket.isOutputShutdown() ){
                    DFracLog.log(2,"DFracSocketedServer["+mnPort+"]: connection lost...");
                    mbConnected=false;
                    break;
                }

                //wait 4 thread some time...

                if (moFracPiece!=null){
                    try{
                        moFracPiece.join(20);
                        if (moFracPiece.isReady()){
                            //calculatin done... send whole
                            moOOS.writeObject(moFracPiece);
                            moFracPiece=null;
                            DFracLog.log(3,"DFracSocketedServer["+mnPort+"]: data sent...");

                        }else{
                            //still calcing... send only status
                            moOOS.writeByte(moFracPiece.getProgress());
                            DFracLog.log(3,"DFracSocketedServer["+mnPort+"]: progress update sent:"+moFracPiece.getProgress());
                        }
                        moOOS.flush();
                    }catch (Exception ex){
                        DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:mainLoop: Error sending Data: "+ex.getMessage());
                        moFracPiece=null;
                    }

                    if (moConnSocket.isClosed() || moConnSocket.isInputShutdown() ||
                    moConnSocket.isOutputShutdown() ){
                        DFracLog.log(2,"DFracSocketedServer["+mnPort+"]: connection lost...");
                        mbConnected=false;
                        break;
                    }
                }
                //and look if theres new data...
                try{
                    if (moConnSocket.getInputStream().available() > 4){
                        if (moOIS==null)moOIS  = new ObjectInputStream(moConnSocket.getInputStream());
                        ReadAndInterpretData();
                    }
                }catch(IOException e){
                    DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:Mainloop:Interpret: IOError reading data:closing connxion: "+e.getMessage());
                    this.mbConnected=false;
                }catch(Exception e){
                    DFracLog.log(2,"DFracSocketedServer["+mnPort+"]:Mainloop: Error reading data: "+e.getMessage());
                }
                //and wait some time
                try{ Thread.sleep(50); }catch(Exception e){ }

            }//mbConn

        }//main program loop
    }

    private void ReadAndInterpretData(){
        ClFracPiece nFP=null;

        try{
            nFP = (ClFracPiece) moOIS.readObject();
        }catch(IOException e){
            DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:Mainloop:Interpret: IOError reading data:closing connxion: "+e.getMessage());
            this.mbConnected=false;
        }catch(Exception e){
            DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:Mainloop:Interpret: Error reading data: "+e.getMessage());
        }

        DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:Interpret: got data, analyzing...");
        if (nFP!=null){
            switch(nFP.mnMode){
                case IFracPiece.STATE_PING:
                    DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:Interpret: state request(PING), sending response...");
                    try{
                        moOOS.writeObject(new ClFracPiece(IFracPiece.STATE_PONG,MAJOR,MINOR));
                        moOOS.flush();

                    }catch(IOException e){
                        DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:PING: Error sending state response: "+e.getMessage());
                        mbConnected=false;
                    }
                    break;

                case IFracPiece.STOP:
                    DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:STOP: stop request,killing thread...");
                    try{
                        if (moFracPiece != null){
                            moFracPiece.interrupt();
                            moFracPiece=null;
                        }
                    }catch(Exception e){
                        DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:STOP: Error sending data: "+e.getMessage());
                        moFracPiece=null;
                        mbConnected=false;
                    }
                    break;

                case IFracPiece.CALC:
                    DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:CALC: calc request, starting thread...");
                    if (moFracPiece != null) moFracPiece.interrupt();
                    moFracPiece = nFP;
                    moFracPiece.start();
                    break;

                case IFracPiece.KILL:
                    try{
                        DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:KILL: kill request, killing thread...");
                        if (moFracPiece != null){
                            moFracPiece.interrupt();
                            moOOS.writeObject(moFracPiece);
                            moOOS.flush();
                            moFracPiece=null;
                        }

                        moConnSocket.close();
                        moSSocket.close();
                        DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:KILL: closed Sockets, exiting...");
                        System.exit(0);
                    }catch(Exception e){
                        DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:KILL: Error, exiting...");
                        System.exit(1);
                    }
                    break;

                case IFracPiece.CLOSE:
                    try{
                        DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:CLOSE: close request, closing connxion...");
                        if (moFracPiece != null){
                            moFracPiece.interrupt();
                            moFracPiece=null;
                        }

                        moConnSocket.close();
                        moConnSocket=null;
                        moOIS=null;
                        DFracLog.log(3,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:CLOSE: closed Sockets...");
                        mbConnected=false;
                    }catch(Exception e){
                        DFracLog.log(1,"DFracSocketedServer["+mnPort+"]:mainLoop:InterPret:CLOSE: Error while closing");
                        mbConnected=false;
                    }
                    break;

            }

        }
    }
}
